#include "TDC_GP22.h"

uint8_t ReadFlag = 0;
uint8_t TDC_INTFlag = 0;

uint8_t RES0Buf[4];
uint8_t RES1Buf[4];
uint8_t RES2Buf[4];
uint8_t RES3Buf[4];
uint8_t STATBuf[2];
uint8_t REG1Buf[1];
uint8_t PW1STBuf[1];

void Delay(__IO uint16_t nCount)
{
   /* Decrement nCount value */
   while (nCount != 0)
   {
    nCount--;
   } 
}

void SpiInit()
{
    CLK_PeripheralClockConfig(CLK_Peripheral_SPI1, ENABLE);
    GPIO_ExternalPullUpConfig(SPI_SCK_PORT, SPI_SCK_PIN, ENABLE);
    GPIO_ExternalPullUpConfig(SPI_MOSI_PORT, SPI_MOSI_PIN, ENABLE);
    GPIO_ExternalPullUpConfig(SPI_MISO_PORT, SPI_MISO_PIN, ENABLE);
    SPI_Init(SPI1, SPI_FirstBit_MSB, SPI_BaudRatePrescaler_2, SPI_Mode_Master, SPI_CPOL_Low, SPI_CPHA_2Edge, SPI_Direction_2Lines_FullDuplex, SPI_NSS_Soft, 0x07);
    SPI_Cmd(SPI1, ENABLE);

    SPI_Disable;
    Delay(0xFFF);
    
    ShutDownTDC();
}

void WriteSPIData(uint8_t *data,uint16_t dataLen)
{
    for (uint16_t i=0;i<dataLen;i++)
    {
        while(!SPI_GetFlagStatus(SPI1, SPI_FLAG_TXE));
        SPI_SendData(SPI1, *data++);
    } 
}

void ReadSPIData(uint8_t *data,uint16_t dataLen)
{
    for (uint16_t i=0;i<dataLen;i++)
    {
        while(!SPI_GetFlagStatus(SPI1, SPI_FLAG_TXE));
        SPI_SendData(SPI1, 0xff);
        SPI_ReceiveData(SPI1);
        (*data++) = SPI_ReceiveData(SPI1);
    }
}

void OpCodeGP22(uint8_t data)
{
    uint8_t temp[1];
    if(ReadFlag == 1)
    {
        temp[0] = data;
        WriteSPIData((uint8_t *)&temp[0],1);
    }
    else if(ReadFlag == 0)
    {
        SPI_Enable;
        temp[0] = data;
        WriteSPIData((uint8_t *)&temp[0],1);
        SPI_Disable;
        Delay(0xFF);
    }
}

void ConfigGP22(uint8_t addr, uint32_t data)
{
    SPI_Enable;
    uint8_t temp[5];
    temp[4] = data & 0xff;
    temp[3] = (data >> 8) & 0xff;
    temp[2] = (data >> 16) & 0xff;
    temp[1] = (data >> 24) & 0xff;
    temp[0] = addr;
    WriteSPIData((uint8_t *)&temp[0],5);
    SPI_Disable;
    Delay(0xF);
}

void ReadGP22(uint8_t *data,uint16_t Symbol)
{
    ReadFlag = 1;
    SPI_Enable;
    Delay(0xF);
    switch(Symbol)
    {
    case ReadRES_0:
      OpCodeGP22(Symbol);
      ReadSPIData((uint8_t *)&data[0],4);    
      break;
      
    case ReadRES_1:
      OpCodeGP22(Symbol);
      ReadSPIData((uint8_t *)&data[0],4);   
      break;
      
    case ReadRES_2:
      OpCodeGP22(Symbol);
      ReadSPIData((uint8_t *)&data[0],4);   
      break;
      
    case ReadRES_3:
      OpCodeGP22(Symbol);
      ReadSPIData((uint8_t *)&data[0],4);    
      break;
      
    case ReadSTAT:
      OpCodeGP22(Symbol);
      ReadSPIData((uint8_t *)&data[0],2);  
      break;
      
    case ReadREG_1:
      OpCodeGP22(Symbol);
      ReadSPIData((uint8_t *)&data[0],1);
      break;
      
    case ReadPW1ST:
      OpCodeGP22(Symbol);
      ReadSPIData((uint8_t *)&data[0],1);
      break;
      
    default:break;
    }
    ReadFlag = 0;
    SPI_Disable;
    Delay(0xF);
}

void StartGP22Fire(uint8_t Mod_Sel)
{
    TDC_Disable;
    TDC_Enable;
    OPA_Enable;
    GPIO_Init(TDC_INT_PORT, TDC_INT_PIN, GPIO_Mode_In_FL_IT);//GP22 INT
    EXTI_SetPinSensitivity(EXTI_Pin_6, EXTI_Trigger_Falling);
    Delay(0xFF);
    
    //TDC POR
    OpCodeGP22(GP22_POR);
    
    ConfigGP22(0x80,0x430BE800);    //
    ConfigGP22(0x81,0x21444000);    //
    ConfigGP22(0x82,0xA0138800);    //
    ConfigGP22(0x83,0xD0A24800);    //
    ConfigGP22(0x84,0x10004000);    //
    if(Mod_Sel == FireUpMod)
    {
        TDC_FireUp;
        ConfigGP22(0x85,0x20000000);    //FIRE UP
    }
    else if(Mod_Sel == FireDownMod)
    {
        TDC_FireDown;
        ConfigGP22(0x85,0x40000000);    //FIRE DOWN
    }
    ConfigGP22(0x86,0xC0C06100);    // 
    
    //TDC Init
    OpCodeGP22(GP22_Init);
    //TDC TOF
    OpCodeGP22(GP22_Start_TOF);
}

void StartGP22Temp()
{
    TDC_Disable;
    TDC_Enable;
    GPIO_Init(TDC_INT_PORT, TDC_INT_PIN, GPIO_Mode_In_FL_IT);//GP22 INT
    EXTI_SetPinSensitivity(EXTI_Pin_6, EXTI_Trigger_Falling);
    Delay(0xFF);
    
    //TDC POR
    OpCodeGP22(GP22_POR);
    
    ConfigGP22(0x80,0x430BE800);    //
    ConfigGP22(0x81,0x21444000);    //
    ConfigGP22(0x82,0xA0138800);    //
    ConfigGP22(0x83,0xD0A24800);    //
    ConfigGP22(0x84,0x10004000);    //
    ConfigGP22(0x85,0x20000000);    //FIRE UP
    ConfigGP22(0x86,0xC0C06100);    // 
    
    //TDC Init
    OpCodeGP22(GP22_Init);
    //TDC TOF
    OpCodeGP22(GP22_Start_Temp);
}

void ReadGP22Results()
{
    ReadGP22((uint8_t *)&RES0Buf[0],ReadRES_0);
    ReadGP22((uint8_t *)&RES1Buf[0],ReadRES_1);
    ReadGP22((uint8_t *)&RES2Buf[0],ReadRES_2);
    ReadGP22((uint8_t *)&RES3Buf[0],ReadRES_3);
    
//    ReadGP22((uint8_t *)&STATBuf[0],ReadSTAT);
//    ReadGP22((uint8_t *)&REG1Buf[0],ReadREG_1);
//    ReadGP22((uint8_t *)&PW1STBuf[0],ReadPW1ST);
    
    nop();
}

void ShutDownTDC()
{
    TDC_Disable;
    TDC_Enable;
    GPIO_Init(TDC_INT_PORT, TDC_INT_PIN, GPIO_Mode_Out_OD_HiZ_Slow);
    GPIO_WriteBit(TDC_INT_PORT, TDC_INT_PIN, SET);
    Delay(0xFF);
    OpCodeGP22(GP22_POR);
    ConfigGP22(0x80,0x430BE800);    //
    ConfigGP22(0x81,0x21444000);    //
    ConfigGP22(0x86,0xC0C06100);    // 
    OpCodeGP22(GP22_Init);
    OpCodeGP22(GP22_Start_Temp);
   // GPIO_Init(TDC_RST_PORT, SPI_NSS_PIN | TDC_RST_PIN, GPIO_Mode_Out_PP_High_Slow);
    nop();
}